
# q-expansion of newform 1280.4.d.g, downloaded from the LMFDB on 02 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1280
weight = 4
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[511, [1, 0]], [261, [-1, 0]], [257, [1, 0]]]
aps_data = [[0, 0], [0, 8], [0, -5], [-4, 0], [0, 12], [0, -58], [66, 0], [0, 100], [132, 0], [0, -90], [-152, 0], [0, 34], [438, 0], [0, 32], [204, 0], [0, -222], [0, 420], [0, 902], [0, 1024], [432, 0], [-362, 0], [160, 0], [0, -72], [-810, 0], [1106, 0], [0, 258], [-988, 0], [0, -24], [0, 950], [-1038, 0], [124, 0], [0, -132], [1254, 0], [0, -2860], [0, -750], [-448, 0], [0, 2246], [0, 568], [-1524, 0], [0, 3702], [0, -3180], [0, 2098], [-4392, 0], [-2158, 0], [0, 1074], [2840, 0], [0, 2668], [-1772, 0], [0, 2784], [0, -350], [-1962, 0], [4320, 0], [-478, 0], [0, 2652], [-2334, 0], [-3948, 0], [0, 1590], [-4952, 0], [0, -1646], [1158, 0], [0, 6992], [0, 258], [0, 8944], [1392, 0], [5878, 0], [0, 10326], [0, -4228], [1106, 0], [0, 9336], [0, -11770], [8322, 0], [10680, 0], [5884, 0], [0, 2098], [0, 3860], [9588, 0], [0, 13410], [0, -13114], [-5838, 0], [-9530, 0], [0, -7260], [0, -12062], [13608, 0], [-3838, 0], [7400, 0], [0, 8352], [10770, 0], [6694, 0], [0, -3018], [-14492, 0], [0, -7776], [13680, 0], [7916, 0], [0, 13932], [0, 8260], [-11148, 0], [0, -9690], [16038, 0], [0, 992], [0, 7142], [0, -7616], [0, -10314], [0, 7128], [-2010, 0], [0, -23188], [22466, 0], [0, 22776], [-21198, 0], [15960, 0], [-5882, 0], [-8516, 0], [0, -8462], [11094, 0], [0, 2180], [-26848, 0], [26322, 0], [0, 10168], [-23604, 0], [0, 16422], [0, 26100], [0, 3058], [10802, 0], [0, 10674], [0, -28608], [0, 2428], [0, -6618], [0, -20510], [-31680, 0], [13196, 0], [0, 8102], [0, 12580], [29892, 0], [40408, 0], [0, -32366], [17238, 0], [10850, 0], [0, -9102], [0, 25504], [0, 14166], [-33210, 0], [0, 39212], [0, -6222], [31172, 0], [0, -264], [0, -29050], [-21720, 0], [0, 6658], [13974, 0], [0, 23780], [12228, 0], [0, 11606], [-32958, 0], [0, -8072], [15756, 0], [0, 18776], [20568, 0], [-6280, 0], [-20430, 0], [-8906, 0], [0, -17418], [0, 2544], [-15402, 0], [-49444, 0], [0, -25188], [2946, 0], [15012, 0], [5128, 0], [0, 49714], [-16270, 0], [0, 21378], [0, 1620], [0, -6298], [45792, 0], [27958, 0], [-35600, 0], [-50730, 0], [0, -6868], [0, -21342], [-12748, 0], [0, -12010], [-2996, 0], [0, -25092], [0, -67502], [-52266, 0], [-56652, 0], [0, -17550], [0, -51514], [0, -49832], [-3050, 0], [-30552, 0], [-10558, 0], [0, -55728], [0, -15332], [0, 56742], [0, -8496], [22518, 0], [2882, 0], [0, -55978], [35586, 0], [-14268, 0], [0, -64650], [-3512, 0], [0, -25886], [75170, 0], [0, -81900], [0, -1914], [-13760, 0], [0, -25992], [-2490, 0], [0, -33508], [76706, 0], [0, 5778], [-49948, 0], [0, 20616], [19080, 0], [1078, 0], [-58676, 0], [18162, 0], [50556, 0], [0, -38058], [0, 31618], [-23560, 0], [-55710, 0], [10708, 0], [0, -31296], [0, -29390], [36678, 0], [-103680, 0], [-58564, 0], [0, -81828], [0, -57658], [0, -81020], [44248, 0], [-96522, 0], [0, -9808], [61164, 0], [-6910, 0], [0, 15138], [0, -13740], [59232, 0], [0, 22488], [0, 26012], [-101788, 0], [0, 26390], [-101358, 0], [73320, 0], [36604, 0], [0, -1332], [0, -32140], [-91932, 0], [0, -68074], [97602, 0], [-117684, 0], [19510, 0], [0, -90378], [0, 99780], [0, -97742], [0, 97976], [0, -66846], [69334, 0], [-21212, 0], [0, -59136], [0, 39490], [0, -13018], [63186, 0], [0, -9740], [0, 34230], [-42, 0], [0, 44192], [0, -34302], [0, -85738], [0, 132064], [96838, 0], [115360, 0], [41426, 0], [-39148, 0], [0, 32136], [0, -33850], [-28682, 0], [0, -34932], [-58812, 0], [25472, 0], [-86964, 0], [0, -76862], [0, -46024], [108888, 0], [114242, 0], [0, 50274], [44120, 0], [-111630, 0], [0, 120822], [0, -11136], [-93402, 0], [0, -94548], [0, 98102], [0, 45510], [-100952, 0], [0, 151698], [0, -151020], [0, -32816], [152998, 0], [0, -128874], [-37760, 0]]
